# SPDX-License-Identifier: GPL-3.0-only
# Copyright (C) 2022 Michał Góral

import os
import re


def _grep(path, pattern, flags=0, linelimit=15):
    with open(path, encoding="utf-8") as f:
        for i, line in enumerate(f):
            line = line.strip()
            m = re.match(pattern, line, flags)
            if m:
                return m
            if i >= linelimit:
                break
    return None


def _read_org(path):
    m = _grep(path, r"#\+TITLE: *(.+)", flags=re.IGNORECASE, linelimit=2)
    if not m:
        return ""
    return m.group(1)


def _read_md(path):
    m = _grep(path, r"^(?:title: *|# *)(.+)", linelimit=4)
    if not m:
        return ""
    title = m.group(1).strip("\"' ")
    return title.replace("\\''", "'")


def _read_adoc(path):
    m = _grep(path, r"= +(.+)", linelimit=20)
    if not m:
        return ""
    return m.group(1)


def _basename(path):
    return os.path.basename(path)


def convert_to_canonical(title):
    def conv_word(word):
        return word.replace("'", "").replace('"', "").lower()

    delims = """[ ().?!,:;/-]+"""
    words = re.split(delims, title)
    return "-".join(conv_word(w) for w in words if w)


def get_title(path):
    supported = {
        ".org": _read_org,
        ".md": _read_md,
        ".adoc": _read_adoc,
        ".lyx": _basename,
        ".rem": _basename,
    }

    _, ext = os.path.splitext(path)
    if ext in supported:
        return supported[ext](path)
    return None


if __name__ == "__main__":
    import sys
    for arg in sys.argv[1:]:
        print(get_title(arg))
